@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,10 +22,9 @@
 #include "Instance.hxx"
 #include "CommandLine.hxx"
 #include "PlaylistFile.hxx"
-#include "PlaylistGlobal.hxx"
 #include "MusicChunk.hxx"
 #include "StateFile.hxx"
-#include "PlayerThread.hxx"
+#include "player/Thread.hxx"
 #include "Mapper.hxx"
 #include "Permission.hxx"
 #include "Listen.hxx"
@@ -34,11 +33,10 @@
 #include "command/AllCommands.hxx"
 #include "Partition.hxx"
 #include "tag/TagConfig.hxx"
-#include "ReplayGainConfig.hxx"
+#include "ReplayGainGlobal.hxx"
 #include "Idle.hxx"
 #include "Log.hxx"
 #include "LogInit.hxx"
-#include "GlobalEvents.hxx"
 #include "input/Init.hxx"
 #include "event/Loop.hxx"
 #include "IOThread.hxx"
@@ -47,22 +45,22 @@
 #include "playlist/PlaylistRegistry.hxx"
 #include "zeroconf/ZeroconfGlue.hxx"
 #include "decoder/DecoderList.hxx"
-#include "AudioConfig.hxx"
+#include "AudioParser.hxx"
 #include "pcm/PcmConvert.hxx"
 #include "unix/SignalHandlers.hxx"
-#include "unix/Daemon.hxx"
 #include "system/FatalError.hxx"
-#include "util/UriUtil.hxx"
-#include "util/Error.hxx"
-#include "thread/Id.hxx"
 #include "thread/Slack.hxx"
 #include "lib/icu/Init.hxx"
 #include "config/ConfigGlobal.hxx"
-#include "config/ConfigData.hxx"
+#include "config/Param.hxx"
 #include "config/ConfigDefaults.hxx"
 #include "config/ConfigOption.hxx"
 #include "config/ConfigError.hxx"
-#include "Stats.hxx"
+#include "util/RuntimeError.hxx"
+
+#ifdef ENABLE_DAEMON
+#include "unix/Daemon.hxx"
+#endif
 
 #ifdef ENABLE_DATABASE
 #include "db/update/Service.hxx"
@@ -98,8 +96,8 @@
 #include "org_musicpd_Bridge.h"
 #endif
 
-#ifdef HAVE_GLIB
-#include <glib.h>
+#ifdef ENABLE_SYSTEMD_DAEMON
+#include <systemd/sd-daemon.h>
 #endif
 
 #include <stdlib.h>
@@ -119,7 +117,21 @@
 
 #include <limits.h>
 
-static constexpr unsigned DEFAULT_BUFFER_SIZE = 4096;
+static constexpr size_t KILOBYTE = 1024;
+static constexpr size_t MEGABYTE = 1024 * KILOBYTE;
+
+static constexpr size_t DEFAULT_BUFFER_SIZE = 4 * MEGABYTE;
+
+static
+#if GCC_OLDER_THAN(5,0)
+/* gcc 4.x has no "constexpr" for std::max() */
+const
+#else
+constexpr
+#endif
+size_t MIN_BUFFER_SIZE = std::max(CHUNK_SIZE * 32,
+				  64 * KILOBYTE);
+
 static constexpr unsigned DEFAULT_BUFFER_BEFORE_PLAY = 10;
 
 #ifdef ANDROID
@@ -128,55 +140,49 @@ Context *context;
 
 Instance *instance;
 
-static StateFile *state_file;
+struct Config {
+	ReplayGainConfig replay_gain;
+};
 
-#ifndef ANDROID
-
-static bool
-glue_daemonize_init(const struct options *options, Error &error)
+static Config
+LoadConfig()
 {
-	auto pid_file = config_get_path(CONF_PID_FILE, error);
-	if (pid_file.IsNull() && error.IsDefined())
-		return false;
+	return {LoadReplayGainConfig()};
+}
 
-	daemonize_init(config_get_string(CONF_USER, nullptr),
-		       config_get_string(CONF_GROUP, nullptr),
-		       std::move(pid_file));
+#ifdef ENABLE_DAEMON
+
+static void
+glue_daemonize_init(const struct options *options)
+{
+	daemonize_init(config_get_string(ConfigOption::USER, nullptr),
+		       config_get_string(ConfigOption::GROUP, nullptr),
+		       config_get_path(ConfigOption::PID_FILE));
 
 	if (options->kill)
 		daemonize_kill();
-
-	return true;
 }
 
 #endif
 
-static bool
-glue_mapper_init(Error &error)
+static void
+glue_mapper_init()
 {
-	auto playlist_dir = config_get_path(CONF_PLAYLIST_DIR, error);
-	if (playlist_dir.IsNull() && error.IsDefined())
-		return false;
-
-	mapper_init(std::move(playlist_dir));
-	return true;
+	mapper_init(config_get_path(ConfigOption::PLAYLIST_DIR));
 }
 
 #ifdef ENABLE_DATABASE
 
-static bool
-InitStorage(Error &error)
+static void
+InitStorage()
 {
-	Storage *storage = CreateConfiguredStorage(io_thread_get(), error);
+	Storage *storage = CreateConfiguredStorage(io_thread_get());
 	if (storage == nullptr)
-		return !error.IsDefined();
-
-	assert(!error.IsDefined());
+		return;
 
 	CompositeStorage *composite = new CompositeStorage();
 	instance->storage = composite;
 	composite->Mount("", storage);
-	return true;
 }
 
 /**
@@ -187,20 +193,13 @@ InitStorage(Error &error)
 static bool
 glue_db_init_and_load(void)
 {
-	Error error;
 	instance->database =
-		CreateConfiguredDatabase(*instance->event_loop, *instance,
-					 error);
-	if (instance->database == nullptr) {
-		if (error.IsDefined())
-			FatalError(error);
-		else
-			return true;
-	}
+		CreateConfiguredDatabase(instance->event_loop, *instance);
+	if (instance->database == nullptr)
+		return true;
 
 	if (instance->database->GetPlugin().flags & DatabasePlugin::FLAG_REQUIRE_STORAGE) {
-		if (!InitStorage(error))
-			FatalError(error);
+		InitStorage();
 
 		if (instance->storage == nullptr) {
 			delete instance->database;
@@ -217,14 +216,17 @@ glue_db_init_and_load(void)
 				   "because the database does not need it");
 	}
 
-	if (!instance->database->Open(error))
-		FatalError(error);
+	try {
+		instance->database->Open();
+	} catch (...) {
+		std::throw_with_nested(std::runtime_error("Failed to open database plugin"));
+	}
 
 	if (!instance->database->IsPlugin(simple_db_plugin))
 		return true;
 
 	SimpleDatabase &db = *(SimpleDatabase *)instance->database;
-	instance->update = new UpdateService(*instance->event_loop, db,
+	instance->update = new UpdateService(instance->event_loop, db,
 					     static_cast<CompositeStorage &>(*instance->storage),
 					     *instance);
 
@@ -245,49 +247,41 @@ InitDatabaseAndStorage()
  * Configure and initialize the sticker subsystem.
  */
 static void
-glue_sticker_init(void)
+glue_sticker_init()
 {
 #ifdef ENABLE_SQLITE
-	Error error;
-	auto sticker_file = config_get_path(CONF_STICKER_FILE, error);
-	if (sticker_file.IsNull()) {
-		if (error.IsDefined())
-			FatalError(error);
+	auto sticker_file = config_get_path(ConfigOption::STICKER_FILE);
+	if (sticker_file.IsNull())
 		return;
-	}
 
-	if (!sticker_global_init(std::move(sticker_file), error))
-		FatalError(error);
+	sticker_global_init(std::move(sticker_file));
 #endif
 }
 
-static bool
-glue_state_file_init(Error &error)
+static void
+glue_state_file_init()
 {
-	auto path_fs = config_get_path(CONF_STATE_FILE, error);
+	auto path_fs = config_get_path(ConfigOption::STATE_FILE);
 	if (path_fs.IsNull()) {
-		if (error.IsDefined())
-			return false;
-
 #ifdef ANDROID
 		const auto cache_dir = GetUserCacheDir();
 		if (cache_dir.IsNull())
-			return true;
+			return;
 
 		path_fs = AllocatedPath::Build(cache_dir, "state");
 #else
-		return true;
+		return;
 #endif
 	}
 
-	unsigned interval = config_get_unsigned(CONF_STATE_FILE_INTERVAL,
-						StateFile::DEFAULT_INTERVAL);
+	const auto interval =
+		config_get_unsigned(ConfigOption::STATE_FILE_INTERVAL,
+				    StateFile::DEFAULT_INTERVAL);
 
-	state_file = new StateFile(std::move(path_fs), interval,
-				   *instance->partition,
-				   *instance->event_loop);
-	state_file->Read();
-	return true;
+	instance->state_file = new StateFile(std::move(path_fs), interval,
+					     *instance->partition,
+					     instance->event_loop);
+	instance->state_file->Read();
 }
 
 /**
@@ -313,12 +307,12 @@ static void winsock_init(void)
  * Initialize the decoder and player core, including the music pipe.
  */
 static void
-initialize_decoder_and_player(void)
+initialize_decoder_and_player(const ReplayGainConfig &replay_gain_config)
 {
-	const struct config_param *param;
+	const ConfigParam *param;
 
 	size_t buffer_size;
-	param = config_get_param(CONF_AUDIO_BUFFER_SIZE);
+	param = config_get_param(ConfigOption::AUDIO_BUFFER_SIZE);
 	if (param != nullptr) {
 		char *test;
 		long tmp = strtol(param->value.c_str(), &test, 10);
@@ -326,12 +320,17 @@ initialize_decoder_and_player(void)
 			FormatFatalError("buffer size \"%s\" is not a "
 					 "positive integer, line %i",
 					 param->value.c_str(), param->line);
-		buffer_size = tmp;
+		buffer_size = tmp * KILOBYTE;
+
+		if (buffer_size < MIN_BUFFER_SIZE) {
+			FormatWarning(config_domain, "buffer size %lu is too small, using %lu bytes instead",
+				      (unsigned long)buffer_size,
+				      (unsigned long)MIN_BUFFER_SIZE);
+			buffer_size = MIN_BUFFER_SIZE;
+		}
 	} else
 		buffer_size = DEFAULT_BUFFER_SIZE;
 
-	buffer_size *= 1024;
-
 	const unsigned buffered_chunks = buffer_size / CHUNK_SIZE;
 
 	if (buffered_chunks >= 1 << 15)
@@ -339,7 +338,7 @@ initialize_decoder_and_player(void)
 				 (unsigned long)buffer_size);
 
 	float perc;
-	param = config_get_param(CONF_BUFFER_BEFORE_PLAY);
+	param = config_get_param(ConfigOption::BUFFER_BEFORE_PLAY);
 	if (param != nullptr) {
 		char *test;
 		perc = strtod(param->value.c_str(), &test);
@@ -349,6 +348,19 @@ initialize_decoder_and_player(void)
 					 "than 100 percent, line %i",
 					 param->value.c_str(), param->line);
 		}
+
+		if (perc > 80) {
+			/* this upper limit should avoid deadlocks
+			   which can occur because the DecoderThread
+			   cannot ever fill the music buffer to
+			   exactly 100%; a few chunks always need to
+			   be available to generate silence in
+			   Player::SendSilence() */
+			FormatError(config_domain,
+				    "buffer_before_play is too large (%f%%), capping at 80%%; please fix your configuration",
+				    perc);
+			perc = 80;
+		}
 	} else
 		perc = DEFAULT_BUFFER_BEFORE_PLAY;
 
@@ -357,45 +369,51 @@ initialize_decoder_and_player(void)
 		buffered_before_play = buffered_chunks;
 
 	const unsigned max_length =
-		config_get_positive(CONF_MAX_PLAYLIST_LENGTH,
+		config_get_positive(ConfigOption::MAX_PLAYLIST_LENGTH,
 				    DEFAULT_PLAYLIST_MAX_LENGTH);
 
+	AudioFormat configured_audio_format = AudioFormat::Undefined();
+	param = config_get_param(ConfigOption::AUDIO_OUTPUT_FORMAT);
+	if (param != nullptr) {
+		try {
+			configured_audio_format = ParseAudioFormat(param->value.c_str(),
+								   true);
+		} catch (const std::runtime_error &) {
+			std::throw_with_nested(FormatRuntimeError("error parsing line %i",
+								  param->line));
+		}
+	}
+
 	instance->partition = new Partition(*instance,
 					    max_length,
 					    buffered_chunks,
-					    buffered_before_play);
+					    buffered_before_play,
+					    configured_audio_format,
+					    replay_gain_config);
+
+	try {
+		param = config_get_param(ConfigOption::REPLAYGAIN);
+		if (param != nullptr)
+			instance->partition->replay_gain_mode =
+				FromString(param->value.c_str());
+	} catch (...) {
+		std::throw_with_nested(FormatRuntimeError("Failed to parse line %i",
+							  param->line));
+	}
 }
 
-/**
- * Handler for GlobalEvents::IDLE.
- */
-static void
-idle_event_emitted(void)
+void
+Instance::OnIdle(unsigned flags)
 {
 	/* send "idle" notifications to all subscribed
 	   clients */
-	unsigned flags = idle_get();
-	if (flags != 0)
-		instance->client_list->IdleAdd(flags);
+	client_list->IdleAdd(flags);
 
 	if (flags & (IDLE_PLAYLIST|IDLE_PLAYER|IDLE_MIXER|IDLE_OUTPUT) &&
 	    state_file != nullptr)
 		state_file->CheckModified();
 }
 
-#ifdef WIN32
-
-/**
- * Handler for GlobalEvents::SHUTDOWN.
- */
-static void
-shutdown_event_emitted(void)
-{
-	instance->event_loop->Break();
-}
-
-#endif
-
 #ifndef ANDROID
 
 int main(int argc, char *argv[])
@@ -409,38 +427,29 @@ int main(int argc, char *argv[])
 
 #endif
 
-static int mpd_main_after_fork(struct options);
+static int
+mpd_main_after_fork(const Config &config);
 
 #ifdef ANDROID
 static inline
 #endif
 int mpd_main(int argc, char *argv[])
-{
+try {
 	struct options options;
-	Error error;
+
+#ifdef ENABLE_DAEMON
+	daemonize_close_stdin();
+#endif
 
 #ifndef ANDROID
-	daemonize_close_stdin();
-
 #ifdef HAVE_LOCALE_H
 	/* initialize locale */
 	setlocale(LC_CTYPE,"");
-#endif
-
-#ifdef HAVE_GLIB
-	g_set_application_name("Music Player Daemon");
-
-#if !GLIB_CHECK_VERSION(2,32,0)
-	/* enable GLib's thread safety code */
-	g_thread_init(nullptr);
-#endif
+	setlocale(LC_COLLATE, "");
 #endif
 #endif
 
-	if (!IcuInit(error)) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
+	IcuInit();
 
 	winsock_init();
 	io_thread_init();
@@ -450,47 +459,32 @@ int mpd_main(int argc, char *argv[])
 	(void)argc;
 	(void)argv;
 
-	{
-		const auto sdcard = Environment::getExternalStorageDirectory();
-		if (!sdcard.IsNull()) {
-			const auto config_path =
-				AllocatedPath::Build(sdcard, "mpd.conf");
-			if (FileExists(config_path) &&
-			    !ReadConfigFile(config_path, error)) {
-				LogError(error);
-				return EXIT_FAILURE;
-			}
-		}
+	const auto sdcard = Environment::getExternalStorageDirectory();
+	if (!sdcard.IsNull()) {
+		const auto config_path =
+			AllocatedPath::Build(sdcard, "mpd.conf");
+		if (FileExists(config_path))
+			ReadConfigFile(config_path);
 	}
 #else
-	if (!parse_cmdline(argc, argv, &options, error)) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
+	ParseCommandLine(argc, argv, &options);
+#endif
 
-	if (!glue_daemonize_init(&options, error)) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
+	const auto config = LoadConfig();
+
+#ifdef ENABLE_DAEMON
+	glue_daemonize_init(&options);
 #endif
 
-	stats_global_init();
 	TagLoadConfig();
 
-	if (!log_init(options.verbose, options.log_stderr, error)) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
+	log_init(options.verbose, options.log_stderr);
 
 	instance = new Instance();
-	instance->event_loop = new EventLoop();
 
 #ifdef ENABLE_NEIGHBOR_PLUGINS
 	instance->neighbors = new NeighborGlue();
-	if (!instance->neighbors->Init(io_thread_get(), *instance, error)) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
+	instance->neighbors->Init(io_thread_get(), *instance);
 
 	if (instance->neighbors->IsEmpty()) {
 		delete instance->neighbors;
@@ -498,18 +492,15 @@ int mpd_main(int argc, char *argv[])
 	}
 #endif
 
-	const unsigned max_clients = config_get_positive(CONF_MAX_CONN, 10);
+	const unsigned max_clients =
+		config_get_positive(ConfigOption::MAX_CONN, 10);
 	instance->client_list = new ClientList(max_clients);
 
-	initialize_decoder_and_player();
+	initialize_decoder_and_player(config.replay_gain);
 
-	if (!listen_global_init(*instance->event_loop, *instance->partition,
-				error)) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
+	listen_global_init(instance->event_loop, *instance->partition);
 
-#ifndef ANDROID
+#ifdef ENABLE_DAEMON
 	daemonize_set_user();
 	daemonize_begin(options.daemon);
 #endif
@@ -522,43 +513,32 @@ int mpd_main(int argc, char *argv[])
 	   This must be run after forking; if dispatch is called before forking,
 	   the child process will have a broken internal dispatch state. */
 	dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
-		exit(mpd_main_after_fork(options));
+		exit(mpd_main_after_fork(config));
 	});
 	dispatch_main();
 	return EXIT_FAILURE; // unreachable, because dispatch_main never returns
 #else
-	return mpd_main_after_fork(options);
+	return mpd_main_after_fork(config);
 #endif
+} catch (const std::exception &e) {
+	LogError(e);
+	return EXIT_FAILURE;
 }
 
-static int mpd_main_after_fork(struct options options)
-{
-	Error error;
-
-	GlobalEvents::Initialize(*instance->event_loop);
-	GlobalEvents::Register(GlobalEvents::IDLE, idle_event_emitted);
-#ifdef WIN32
-	GlobalEvents::Register(GlobalEvents::SHUTDOWN, shutdown_event_emitted);
-#endif
-
+static int
+mpd_main_after_fork(const Config &config)
+try {
 	ConfigureFS();
 
-	if (!glue_mapper_init(error)) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
+	glue_mapper_init();
 
 	initPermissions();
-	playlist_global_init();
 	spl_global_init();
 #ifdef ENABLE_ARCHIVE
 	archive_plugin_init_all();
 #endif
 
-	if (!pcm_convert_global_init(error)) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
+	pcm_convert_global_init();
 
 	decoder_plugin_init_all();
 
@@ -569,36 +549,34 @@ static int mpd_main_after_fork(struct options options)
 	glue_sticker_init();
 
 	command_init();
-	initAudioConfig();
-	instance->partition->outputs.Configure(*instance->event_loop,
+
+	instance->partition->outputs.Configure(instance->event_loop,
+					       config.replay_gain,
 					       instance->partition->pc);
+	instance->partition->UpdateEffectiveReplayGainMode();
+
 	client_manager_init();
-	replay_gain_global_init();
-
-	if (!input_stream_global_init(error)) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
-
+	input_stream_global_init();
 	playlist_list_global_init();
 
-#ifndef ANDROID
+#ifdef ENABLE_DAEMON
 	daemonize_commit();
+#endif
 
-	setup_log_output(options.log_stderr);
+#ifndef ANDROID
+	setup_log_output();
 
-	SignalHandlersInit(*instance->event_loop);
+	SignalHandlersInit(instance->event_loop);
 #endif
 
 	io_thread_start();
 
 #ifdef ENABLE_NEIGHBOR_PLUGINS
-	if (instance->neighbors != nullptr &&
-	    !instance->neighbors->Open(error))
-		FatalError(error);
+	if (instance->neighbors != nullptr)
+		instance->neighbors->Open();
 #endif
 
-	ZeroconfInit(*instance->event_loop);
+	ZeroconfInit(instance->event_loop);
 
 	StartPlayerThread(instance->partition->pc);
 
@@ -612,22 +590,17 @@ static int mpd_main_after_fork(struct options options)
 	}
 #endif
 
-	if (!glue_state_file_init(error)) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
-
-	instance->partition->outputs.SetReplayGainMode(replay_gain_get_real_mode(instance->partition->playlist.queue.random));
+	glue_state_file_init();
 
 #ifdef ENABLE_DATABASE
-	if (config_get_bool(CONF_AUTO_UPDATE, false)) {
+	if (config_get_bool(ConfigOption::AUTO_UPDATE, false)) {
 #ifdef ENABLE_INOTIFY
 		if (instance->storage != nullptr &&
 		    instance->update != nullptr)
-			mpd_inotify_init(*instance->event_loop,
+			mpd_inotify_init(instance->event_loop,
 					 *instance->storage,
 					 *instance->update,
-					 config_get_unsigned(CONF_AUTO_UPDATE_DEPTH,
+					 config_get_unsigned(ConfigOption::AUTO_UPDATE_DEPTH,
 							     INT_MAX));
 #else
 		FormatWarning(config_domain,
@@ -640,7 +613,7 @@ static int mpd_main_after_fork(struct options options)
 
 	/* enable all audio outputs (if not already done by
 	   playlist_state_restore() */
-	instance->partition->pc.UpdateAudio();
+	instance->partition->pc.LockUpdateAudio();
 
 #ifdef WIN32
 	win32_app_started();
@@ -650,8 +623,12 @@ static int mpd_main_after_fork(struct options options)
 	   a huge value to allow the kernel to reduce CPU wakeups */
 	SetThreadTimerSlackMS(100);
 
+#ifdef ENABLE_SYSTEMD_DAEMON
+	sd_notify(0, "READY=1");
+#endif
+
 	/* run the main loop */
-	instance->event_loop->Run();
+	instance->event_loop.Run();
 
 #ifdef WIN32
 	win32_app_stopping();
@@ -666,9 +643,9 @@ static int mpd_main_after_fork(struct options options)
 		instance->update->CancelAllAsync();
 #endif
 
-	if (state_file != nullptr) {
-		state_file->Write();
-		delete state_file;
+	if (instance->state_file != nullptr) {
+		instance->state_file->Write();
+		delete instance->state_file;
 	}
 
 	instance->partition->pc.Kill();
@@ -698,8 +675,6 @@ static int mpd_main_after_fork(struct options options)
 	sticker_global_finish();
 #endif
 
-	GlobalEvents::Deinitialize();
-
 	playlist_list_global_finish();
 	input_stream_global_finish();
 
@@ -707,6 +682,8 @@ static int mpd_main_after_fork(struct options options)
 	mapper_finish();
 #endif
 
+	DeinitFS();
+
 	delete instance->partition;
 	command_finish();
 	decoder_plugin_deinit_all();
@@ -718,12 +695,13 @@ static int mpd_main_after_fork(struct options options)
 #ifndef ANDROID
 	SignalHandlersFinish();
 #endif
-	delete instance->event_loop;
 	delete instance;
 	instance = nullptr;
-#ifndef ANDROID
+
+#ifdef ENABLE_DAEMON
 	daemonize_finish();
 #endif
+
 #ifdef WIN32
 	WSACleanup();
 #endif
@@ -732,6 +710,9 @@ static int mpd_main_after_fork(struct options options)
 
 	log_deinit();
 	return EXIT_SUCCESS;
+} catch (const std::exception &e) {
+	LogError(e);
+	return EXIT_FAILURE;
 }
 
 #ifdef ANDROID
@@ -757,7 +738,7 @@ JNIEXPORT void JNICALL
 Java_org_musicpd_Bridge_shutdown(JNIEnv *, jclass)
 {
 	if (instance != nullptr)
-		instance->event_loop->Break();
+		instance->Shutdown();
 }
 
 #endif
