@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,79 +23,75 @@
 #include "PlaylistError.hxx"
 #include "queue/Playlist.hxx"
 #include "DetachedSong.hxx"
-#include "SongLoader.hxx"
 #include "Mapper.hxx"
 #include "Idle.hxx"
 #include "fs/AllocatedPath.hxx"
 #include "fs/Traits.hxx"
 #include "fs/FileSystem.hxx"
-#include "util/Alloc.hxx"
+#include "fs/NarrowPath.hxx"
+#include "fs/io/FileOutputStream.hxx"
+#include "fs/io/BufferedOutputStream.hxx"
 #include "util/UriUtil.hxx"
-#include "util/Error.hxx"
-#include "Log.hxx"
 
-#include <string.h>
+#include <stdexcept>
 
 void
-playlist_print_song(FILE *file, const DetachedSong &song)
+playlist_print_song(BufferedOutputStream &os, const DetachedSong &song)
 {
 	const char *uri_utf8 = playlist_saveAbsolutePaths
 		? song.GetRealURI()
 		: song.GetURI();
 
-	const auto uri_fs = AllocatedPath::FromUTF8(uri_utf8);
-	if (!uri_fs.IsNull())
-		fprintf(file, "%s\n", uri_fs.c_str());
+	try {
+		const auto uri_fs = AllocatedPath::FromUTF8Throw(uri_utf8);
+		os.Format("%s\n", NarrowPath(uri_fs).c_str());
+	} catch (const std::runtime_error &) {
+	}
 }
 
 void
-playlist_print_uri(FILE *file, const char *uri)
+playlist_print_uri(BufferedOutputStream &os, const char *uri)
 {
-	auto path =
+	try {
+		auto path =
 #ifdef ENABLE_DATABASE
-		playlist_saveAbsolutePaths && !uri_has_scheme(uri) &&
-		!PathTraitsUTF8::IsAbsolute(uri)
-		? map_uri_fs(uri)
-		:
+			playlist_saveAbsolutePaths && !uri_has_scheme(uri) &&
+			!PathTraitsUTF8::IsAbsolute(uri)
+			? map_uri_fs(uri)
+			:
 #endif
-		AllocatedPath::FromUTF8(uri);
+			AllocatedPath::FromUTF8Throw(uri);
 
-	if (!path.IsNull())
-		fprintf(file, "%s\n", path.c_str());
+		if (!path.IsNull())
+			os.Format("%s\n", NarrowPath(path).c_str());
+	} catch (const std::runtime_error &) {
+	}
 }
 
-PlaylistResult
+void
 spl_save_queue(const char *name_utf8, const Queue &queue)
 {
-	if (map_spl_path().IsNull())
-		return PlaylistResult::DISABLED;
-
-	if (!spl_valid_name(name_utf8))
-		return PlaylistResult::BAD_NAME;
-
-	const auto path_fs = map_spl_utf8_to_fs(name_utf8);
-	if (path_fs.IsNull())
-		return PlaylistResult::BAD_NAME;
+	const auto path_fs = spl_map_to_fs(name_utf8);
+	assert(!path_fs.IsNull());
 
 	if (FileExists(path_fs))
-		return PlaylistResult::LIST_EXISTS;
+		throw PlaylistError(PlaylistResult::LIST_EXISTS,
+				    "Playlist already exists");
 
-	FILE *file = FOpen(path_fs, FOpenMode::WriteText);
-
-	if (file == nullptr)
-		return PlaylistResult::ERRNO;
+	FileOutputStream fos(path_fs);
+	BufferedOutputStream bos(fos);
 
 	for (unsigned i = 0; i < queue.GetLength(); i++)
-		playlist_print_song(file, queue.Get(i));
+		playlist_print_song(bos, queue.Get(i));
 
-	fclose(file);
+	bos.Flush();
+	fos.Commit();
 
 	idle_add(IDLE_STORED_PLAYLIST);
-	return PlaylistResult::SUCCESS;
 }
 
-PlaylistResult
+void
 spl_save_playlist(const char *name_utf8, const playlist &playlist)
 {
-	return spl_save_queue(name_utf8, playlist.queue);
+	spl_save_queue(name_utf8, playlist.queue);
 }
