@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,27 +28,29 @@
 #include "Instance.hxx"
 #include "mixer/Volume.hxx"
 #include "SongLoader.hxx"
-#include "fs/FileSystem.hxx"
 #include "util/Domain.hxx"
 #include "Log.hxx"
 
+#include <exception>
+
 #include <string.h>
 
 static constexpr Domain state_file_domain("state_file");
 
-StateFile::StateFile(AllocatedPath &&_path, unsigned _interval,
+constexpr std::chrono::steady_clock::duration StateFile::DEFAULT_INTERVAL;
+
+StateFile::StateFile(AllocatedPath &&_path,
+		     std::chrono::steady_clock::duration _interval,
 		     Partition &_partition, EventLoop &_loop)
 	:TimeoutMonitor(_loop),
 	 path(std::move(_path)), path_utf8(path.ToUTF8()),
 	 interval(_interval),
-	 partition(_partition),
-	 prev_volume_version(0), prev_output_version(0),
-	 prev_playlist_version(0)
+	 partition(_partition)
 {
 }
 
 void
-StateFile::RememberVersions()
+StateFile::RememberVersions() noexcept
 {
 	prev_volume_version = sw_volume_state_get_hash();
 	prev_output_version = audio_output_state_get_version();
@@ -57,7 +59,7 @@ StateFile::RememberVersions()
 }
 
 bool
-StateFile::IsModified() const
+StateFile::IsModified() const noexcept
 {
 	return prev_volume_version != sw_volume_state_get_hash() ||
 		prev_output_version != audio_output_state_get_version() ||
@@ -73,12 +75,12 @@ StateFile::Write(BufferedOutputStream &os)
 	playlist_state_save(os, partition.playlist, partition.pc);
 }
 
-inline bool
-StateFile::Write(OutputStream &os, Error &error)
+inline void
+StateFile::Write(OutputStream &os)
 {
 	BufferedOutputStream bos(os);
 	Write(bos);
-	return bos.Flush(error);
+	bos.Flush();
 }
 
 void
@@ -87,11 +89,12 @@ StateFile::Write()
 	FormatDebug(state_file_domain,
 		    "Saving state file %s", path_utf8.c_str());
 
-	Error error;
-	FileOutputStream fos(path, error);
-	if (!fos.IsDefined() || !Write(fos, error) || !fos.Commit(error)) {
-		LogError(error);
-		return;
+	try {
+		FileOutputStream fos(path);
+		Write(fos);
+		fos.Commit();
+	} catch (const std::exception &e) {
+		LogError(e);
 	}
 
 	RememberVersions();
@@ -99,17 +102,12 @@ StateFile::Write()
 
 void
 StateFile::Read()
-{
+try {
 	bool success;
 
 	FormatDebug(state_file_domain, "Loading state file %s", path_utf8.c_str());
 
-	Error error;
-	TextFile file(path, error);
-	if (file.HasFailed()) {
-		LogError(error);
-		return;
-	}
+	TextFile file(path);
 
 #ifdef ENABLE_DATABASE
 	const SongLoader song_loader(partition.instance.database,
@@ -132,13 +130,15 @@ StateFile::Read()
 	}
 
 	RememberVersions();
+} catch (const std::exception &e) {
+	LogError(e);
 }
 
 void
 StateFile::CheckModified()
 {
 	if (!IsActive() && IsModified())
-		ScheduleSeconds(interval);
+		Schedule(interval);
 }
 
 void
