@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -21,7 +21,6 @@
 #include "UpnpDatabasePlugin.hxx"
 #include "Directory.hxx"
 #include "Tags.hxx"
-#include "lib/upnp/Domain.hxx"
 #include "lib/upnp/ClientInit.hxx"
 #include "lib/upnp/Discovery.hxx"
 #include "lib/upnp/ContentDirectoryService.hxx"
@@ -33,11 +32,9 @@
 #include "db/LightDirectory.hxx"
 #include "db/LightSong.hxx"
 #include "db/Stats.hxx"
-#include "config/ConfigData.hxx"
+#include "config/Block.hxx"
 #include "tag/TagBuilder.hxx"
 #include "tag/TagTable.hxx"
-#include "util/Error.hxx"
-#include "util/Domain.hxx"
 #include "fs/Traits.hxx"
 #include "Log.hxx"
 #include "SongFilter.hxx"
@@ -78,120 +75,88 @@ public:
 	UpnpDatabase():Database(upnp_db_plugin) {}
 
 	static Database *Create(EventLoop &loop, DatabaseListener &listener,
-				const config_param &param,
-				Error &error);
+				const ConfigBlock &block);
 
-	virtual bool Open(Error &error) override;
-	virtual void Close() override;
-	virtual const LightSong *GetSong(const char *uri_utf8,
-					 Error &error) const override;
+	void Open() override;
+	void Close() override;
+	const LightSong *GetSong(const char *uri_utf8) const override;
 	void ReturnSong(const LightSong *song) const override;
 
-	virtual bool Visit(const DatabaseSelection &selection,
-			   VisitDirectory visit_directory,
-			   VisitSong visit_song,
-			   VisitPlaylist visit_playlist,
-			   Error &error) const override;
+	void Visit(const DatabaseSelection &selection,
+		   VisitDirectory visit_directory,
+		   VisitSong visit_song,
+		   VisitPlaylist visit_playlist) const override;
 
-	virtual bool VisitUniqueTags(const DatabaseSelection &selection,
-				     TagType tag_type, uint32_t group_mask,
-				     VisitTag visit_tag,
-				     Error &error) const override;
+	void VisitUniqueTags(const DatabaseSelection &selection,
+			     TagType tag_type, tag_mask_t group_mask,
+			     VisitTag visit_tag) const override;
+
+	DatabaseStats GetStats(const DatabaseSelection &selection) const override;
 
-	virtual bool GetStats(const DatabaseSelection &selection,
-			      DatabaseStats &stats,
-			      Error &error) const override;
 	time_t GetUpdateStamp() const override {
 		return 0;
 	}
 
-protected:
-	bool Configure(const config_param &param, Error &error);
-
 private:
-	bool VisitServer(const ContentDirectoryService &server,
+	void VisitServer(const ContentDirectoryService &server,
 			 const std::list<std::string> &vpath,
 			 const DatabaseSelection &selection,
 			 VisitDirectory visit_directory,
 			 VisitSong visit_song,
-			 VisitPlaylist visit_playlist,
-			 Error &error) const;
+			 VisitPlaylist visit_playlist) const;
 
 	/**
 	 * Run an UPnP search according to MPD parameters, and
 	 * visit_song the results.
 	 */
-	bool SearchSongs(const ContentDirectoryService &server,
+	void SearchSongs(const ContentDirectoryService &server,
 			 const char *objid,
 			 const DatabaseSelection &selection,
-			 VisitSong visit_song,
-			 Error &error) const;
+			 VisitSong visit_song) const;
 
-	bool SearchSongs(const ContentDirectoryService &server,
-			 const char *objid,
-			 const DatabaseSelection &selection,
-			 UPnPDirContent& dirbuf,
-			 Error &error) const;
+	UPnPDirContent SearchSongs(const ContentDirectoryService &server,
+				   const char *objid,
+				   const DatabaseSelection &selection) const;
 
-	bool Namei(const ContentDirectoryService &server,
-		   const std::list<std::string> &vpath,
-		   UPnPDirObject &dirent,
-		   Error &error) const;
+	UPnPDirObject Namei(const ContentDirectoryService &server,
+			    const std::list<std::string> &vpath) const;
 
 	/**
 	 * Take server and objid, return metadata.
 	 */
-	bool ReadNode(const ContentDirectoryService &server,
-		      const char *objid, UPnPDirObject& dirent,
-		      Error &error) const;
+	UPnPDirObject ReadNode(const ContentDirectoryService &server,
+			       const char *objid) const;
 
 	/**
 	 * Get the path for an object Id. This works much like pwd,
 	 * except easier cause our inodes have a parent id. Not used
 	 * any more actually (see comments in SearchSongs).
 	 */
-	bool BuildPath(const ContentDirectoryService &server,
-		       const UPnPDirObject& dirent, std::string &idpath,
-		       Error &error) const;
+	std::string BuildPath(const ContentDirectoryService &server,
+			      const UPnPDirObject& dirent) const;
 };
 
 Database *
 UpnpDatabase::Create(gcc_unused EventLoop &loop,
 		     gcc_unused DatabaseListener &listener,
-		     const config_param &param, Error &error)
+		     const ConfigBlock &)
 {
-	UpnpDatabase *db = new UpnpDatabase();
-	if (!db->Configure(param, error)) {
-		delete db;
-		return nullptr;
-	}
-
-	/* libupnp loses its ability to receive multicast messages
-	   apparently due to daemonization; using the LazyDatabase
-	   wrapper works around this problem */
-	return db;
+	return new UpnpDatabase();
 }
 
-inline bool
-UpnpDatabase::Configure(const config_param &, Error &)
+void
+UpnpDatabase::Open()
 {
-	return true;
-}
-
-bool
-UpnpDatabase::Open(Error &error)
-{
-	if (!UpnpClientGlobalInit(handle, error))
-		return false;
+	UpnpClientGlobalInit(handle);
 
 	discovery = new UPnPDeviceDirectory(handle);
-	if (!discovery->Start(error)) {
+	try {
+		discovery->Start();
+	} catch (...) {
 		delete discovery;
 		UpnpClientGlobalFinish();
-		return false;
+		throw;
 	}
-
-	return true;
 }
 
 void
@@ -213,28 +178,22 @@ UpnpDatabase::ReturnSong(const LightSong *_song) const
 // Get song info by path. We can receive either the id path, or the titles
 // one
 const LightSong *
-UpnpDatabase::GetSong(const char *uri, Error &error) const
+UpnpDatabase::GetSong(const char *uri) const
 {
-	auto vpath = stringToTokens(uri, "/", true);
-	if (vpath.size() < 2) {
-		error.Format(db_domain, DB_NOT_FOUND, "No such song: %s", uri);
-		return nullptr;
-	}
+	auto vpath = stringToTokens(uri, '/');
+	if (vpath.size() < 2)
+		throw DatabaseError(DatabaseErrorCode::NOT_FOUND,
+				    "No such song");
 
-	ContentDirectoryService server;
-	if (!discovery->getServer(vpath.front().c_str(), server, error))
-		return nullptr;
+	auto server = discovery->GetServer(vpath.front().c_str());
 
 	vpath.pop_front();
 
 	UPnPDirObject dirent;
 	if (vpath.front() != rootid) {
-		if (!Namei(server, vpath, dirent, error))
-			return nullptr;
+		dirent = Namei(server, vpath);
 	} else {
-		if (!ReadNode(server, vpath.back().c_str(), dirent,
-			      error))
-			return nullptr;
+		dirent = ReadNode(server, vpath.back().c_str());
 	}
 
 	return new UpnpSong(std::move(dirent), uri);
@@ -264,23 +223,18 @@ dquote(std::string &out, const char *in)
 
 // Run an UPnP search, according to MPD parameters. Return results as
 // UPnP items
-bool
+UPnPDirContent
 UpnpDatabase::SearchSongs(const ContentDirectoryService &server,
 			  const char *objid,
-			  const DatabaseSelection &selection,
-			  UPnPDirContent &dirbuf,
-			  Error &error) const
+			  const DatabaseSelection &selection) const
 {
 	const SongFilter *filter = selection.filter;
 	if (selection.filter == nullptr)
-		return true;
-
-	std::list<std::string> searchcaps;
-	if (!server.getSearchCapabilities(handle, searchcaps, error))
-		return false;
+		return UPnPDirContent();
 
+	const auto searchcaps = server.getSearchCapabilities(handle);
 	if (searchcaps.empty())
-		return true;
+		return UPnPDirContent();
 
 	std::string cond;
 	for (const auto &item : filter->GetItems()) {
@@ -303,7 +257,7 @@ UpnpDatabase::SearchSongs(const ContentDirectoryService &server,
 					} else {
 						cond += " = ";
 					}
-					dquote(cond, item.GetValue().c_str());
+					dquote(cond, item.GetValue());
 				}
 				cond += ')';
 			}
@@ -339,22 +293,20 @@ UpnpDatabase::SearchSongs(const ContentDirectoryService &server,
 			} else {
 				cond += " = ";
 			}
-			dquote(cond, item.GetValue().c_str());
+			dquote(cond, item.GetValue());
 		}
 	}
 
-	return server.search(handle,
-			     objid, cond.c_str(), dirbuf,
-			     error);
+	return server.search(handle, objid, cond.c_str());
 }
 
-static bool
+static void
 visitSong(const UPnPDirObject &meta, const char *path,
 	  const DatabaseSelection &selection,
-	  VisitSong visit_song, Error& error)
+	  VisitSong visit_song)
 {
 	if (!visit_song)
-		return true;
+		return;
 
 	LightSong song;
 	song.directory = nullptr;
@@ -364,7 +316,8 @@ visitSong(const UPnPDirObject &meta, const char *path,
 	song.mtime = 0;
 	song.start_time = song.end_time = SongTime::zero();
 
-	return !selection.Match(song) || visit_song(song, error);
+	if (selection.Match(song))
+		visit_song(song);
 }
 
 /**
@@ -379,20 +332,16 @@ songPath(const std::string &servername,
 	return servername + "/" + rootid + "/" + objid;
 }
 
-bool
+void
 UpnpDatabase::SearchSongs(const ContentDirectoryService &server,
 			  const char *objid,
 			  const DatabaseSelection &selection,
-			  VisitSong visit_song,
-			  Error &error) const
+			  VisitSong visit_song) const
 {
-	UPnPDirContent dirbuf;
 	if (!visit_song)
-		return true;
-	if (!SearchSongs(server, objid, selection, dirbuf, error))
-		return false;
+		return;
 
-	for (auto &dirent : dirbuf.objects) {
+	for (auto &dirent : SearchSongs(server, objid, selection).objects) {
 		if (dirent.type != UPnPDirObject::Type::ITEM ||
 		    dirent.item_class != UPnPDirObject::ItemClass::MUSIC)
 			continue;
@@ -414,48 +363,32 @@ UpnpDatabase::SearchSongs(const ContentDirectoryService &server,
 		// So we return synthetic and ugly paths based on the object id,
 		// which we later have to detect.
 		const std::string path = songPath(server.getFriendlyName(),
-						  dirent.m_id);
-		if (!visitSong(std::move(dirent), path.c_str(),
-			       selection, visit_song,
-			       error))
-			return false;
+						  dirent.id);
+		visitSong(std::move(dirent), path.c_str(),
+			  selection, visit_song);
 	}
-
-	return true;
 }
 
-bool
+UPnPDirObject
 UpnpDatabase::ReadNode(const ContentDirectoryService &server,
-		       const char *objid, UPnPDirObject &dirent,
-		       Error &error) const
+		       const char *objid) const
 {
-	UPnPDirContent dirbuf;
-	if (!server.getMetadata(handle, objid, dirbuf, error))
-		return false;
+	auto dirbuf = server.getMetadata(handle, objid);
+	if (dirbuf.objects.size() != 1)
+		throw std::runtime_error("Bad resource");
 
-	if (dirbuf.objects.size() == 1) {
-		dirent = std::move(dirbuf.objects.front());
-	} else {
-		error.Format(upnp_domain, "Bad resource");
-		return false;
-	}
-
-	return true;
+	return std::move(dirbuf.objects.front());
 }
 
-bool
+std::string
 UpnpDatabase::BuildPath(const ContentDirectoryService &server,
-			const UPnPDirObject& idirent,
-			std::string &path,
-			Error &error) const
+			const UPnPDirObject& idirent) const
 {
-	const char *pid = idirent.m_id.c_str();
-	path.clear();
-	UPnPDirObject dirent;
+	const char *pid = idirent.id.c_str();
+	std::string path;
 	while (strcmp(pid, rootid) != 0) {
-		if (!ReadNode(server, pid, dirent, error))
-			return false;
-		pid = dirent.m_pid.c_str();
+		auto dirent = ReadNode(server, pid);
+		pid = dirent.parent_id.c_str();
 
 		if (path.empty())
 			path = dirent.name;
@@ -464,70 +397,53 @@ UpnpDatabase::BuildPath(const ContentDirectoryService &server,
 						     path.c_str());
 	}
 
-	path = PathTraitsUTF8::Build(server.getFriendlyName(),
+	return PathTraitsUTF8::Build(server.getFriendlyName(),
 				     path.c_str());
-	return true;
 }
 
 // Take server and internal title pathname and return objid and metadata.
-bool
+UPnPDirObject
 UpnpDatabase::Namei(const ContentDirectoryService &server,
-		    const std::list<std::string> &vpath,
-		    UPnPDirObject &odirent,
-		    Error &error) const
+		    const std::list<std::string> &vpath) const
 {
-	if (vpath.empty()) {
+	if (vpath.empty())
 		// looking for root info
-		if (!ReadNode(server, rootid, odirent, error))
-			return false;
-
-		return true;
-	}
+		return ReadNode(server, rootid);
 
 	std::string objid(rootid);
 
 	// Walk the path elements, read each directory and try to find the next one
 	for (auto i = vpath.begin(), last = std::prev(vpath.end());; ++i) {
-		UPnPDirContent dirbuf;
-		if (!server.readDir(handle, objid.c_str(), dirbuf, error))
-			return false;
+		auto dirbuf = server.readDir(handle, objid.c_str());
 
 		// Look for the name in the sub-container list
 		UPnPDirObject *child = dirbuf.FindObject(i->c_str());
-		if (child == nullptr) {
-			error.Format(db_domain, DB_NOT_FOUND,
-				     "No such object");
-			return false;
-		}
+		if (child == nullptr)
+			throw DatabaseError(DatabaseErrorCode::NOT_FOUND,
+					    "No such object");
 
-		if (i == last) {
-			odirent = std::move(*child);
-			return true;
-		}
+		if (i == last)
+			return std::move(*child);
 
-		if (child->type != UPnPDirObject::Type::CONTAINER) {
-			error.Format(db_domain, DB_NOT_FOUND,
-				     "Not a container");
-			return false;
-		}
+		if (child->type != UPnPDirObject::Type::CONTAINER)
+			throw DatabaseError(DatabaseErrorCode::NOT_FOUND,
+					    "Not a container");
 
-		objid = std::move(child->m_id);
+		objid = std::move(child->id);
 	}
 }
 
-static bool
+static void
 VisitItem(const UPnPDirObject &object, const char *uri,
 	  const DatabaseSelection &selection,
-	  VisitSong visit_song, VisitPlaylist visit_playlist,
-	  Error &error)
+	  VisitSong visit_song, VisitPlaylist visit_playlist)
 {
 	assert(object.type == UPnPDirObject::Type::ITEM);
 
 	switch (object.item_class) {
 	case UPnPDirObject::ItemClass::MUSIC:
-		return !visit_song ||
-			visitSong(object, uri,
-				  selection, visit_song, error);
+		visitSong(object, uri, selection, visit_song);
+		break;
 
 	case UPnPDirObject::ItemClass::PLAYLIST:
 		if (visit_playlist) {
@@ -539,23 +455,19 @@ VisitItem(const UPnPDirObject &object, const char *uri,
 			   see one... */
 		}
 
-		return true;
+		break;
 
 	case UPnPDirObject::ItemClass::UNKNOWN:
-		return true;
+		break;
 	}
-
-	assert(false);
-	gcc_unreachable();
 }
 
-static bool
+static void
 VisitObject(const UPnPDirObject &object, const char *uri,
 	    const DatabaseSelection &selection,
 	    VisitDirectory visit_directory,
 	    VisitSong visit_song,
-	    VisitPlaylist visit_playlist,
-	    Error &error)
+	    VisitPlaylist visit_playlist)
 {
 	switch (object.type) {
 	case UPnPDirObject::Type::UNKNOWN:
@@ -563,29 +475,26 @@ VisitObject(const UPnPDirObject &object, const char *uri,
 		gcc_unreachable();
 
 	case UPnPDirObject::Type::CONTAINER:
-		return !visit_directory ||
-			visit_directory(LightDirectory(uri, 0), error);
+		if (visit_directory)
+			visit_directory(LightDirectory(uri, 0));
+		break;
 
 	case UPnPDirObject::Type::ITEM:
-		return VisitItem(object, uri, selection,
-				 visit_song, visit_playlist,
-				 error);
+		VisitItem(object, uri, selection,
+			  visit_song, visit_playlist);
+		break;
 	}
-
-	assert(false);
-	gcc_unreachable();
 }
 
 // vpath is a parsed and writeable version of selection.uri. There is
 // really just one path parameter.
-bool
+void
 UpnpDatabase::VisitServer(const ContentDirectoryService &server,
 			  const std::list<std::string> &vpath,
 			  const DatabaseSelection &selection,
 			  VisitDirectory visit_directory,
 			  VisitSong visit_song,
-			  VisitPlaylist visit_playlist,
-			  Error &error) const
+			  VisitPlaylist visit_playlist) const
 {
 	/* If the path begins with rootid, we know that this is a
 	   song, not a directory (because that's how we set things
@@ -598,149 +507,115 @@ UpnpDatabase::VisitServer(const ContentDirectoryService &server,
 	if (!vpath.empty() && vpath.front() == rootid) {
 		switch (vpath.size()) {
 		case 1:
-			return true;
+			return;
 
 		case 2:
 			break;
 
 		default:
-			error.Format(db_domain, DB_NOT_FOUND,
-				     "Not found");
-			return false;
+			throw DatabaseError(DatabaseErrorCode::NOT_FOUND,
+					    "Not found");
 		}
 
 		if (visit_song) {
-			UPnPDirObject dirent;
-			if (!ReadNode(server, vpath.back().c_str(), dirent,
-				      error))
-				return false;
+			auto dirent = ReadNode(server, vpath.back().c_str());
 
 			if (dirent.type != UPnPDirObject::Type::ITEM ||
-			    dirent.item_class != UPnPDirObject::ItemClass::MUSIC) {
-				error.Format(db_domain, DB_NOT_FOUND,
-					     "Not found");
-				return false;
-			}
+			    dirent.item_class != UPnPDirObject::ItemClass::MUSIC)
+				throw DatabaseError(DatabaseErrorCode::NOT_FOUND,
+						    "Not found");
 
 			std::string path = songPath(server.getFriendlyName(),
-						    dirent.m_id);
-			if (!visitSong(std::move(dirent), path.c_str(),
-				       selection,
-				       visit_song, error))
-				return false;
+						    dirent.id);
+			visitSong(std::move(dirent), path.c_str(),
+				  selection, visit_song);
 		}
-		return true;
+
+		return;
 	}
 
 	// Translate the target path into an object id and the associated metadata.
-	UPnPDirObject tdirent;
-	if (!Namei(server, vpath, tdirent, error))
-		return false;
+	const auto tdirent = Namei(server, vpath);
 
 	/* If recursive is set, this is a search... No use sending it
 	   if the filter is empty. In this case, we implement limited
 	   recursion (1-deep) here, which will handle the "add dir"
 	   case. */
-	if (selection.recursive && selection.filter)
-		return SearchSongs(server, tdirent.m_id.c_str(), selection,
-				   visit_song, error);
+	if (selection.recursive && selection.filter) {
+		SearchSongs(server, tdirent.id.c_str(), selection, visit_song);
+		return;
+	}
 
 	const char *const base_uri = selection.uri.empty()
 		? server.getFriendlyName()
 		: selection.uri.c_str();
 
 	if (tdirent.type == UPnPDirObject::Type::ITEM) {
-		return VisitItem(tdirent, base_uri,
-				 selection,
-				 visit_song, visit_playlist,
-				 error);
+		VisitItem(tdirent, base_uri,
+			  selection,
+			  visit_song, visit_playlist);
+		return;
 	}
 
 	/* Target was a a container. Visit it. We could read slices
 	   and loop here, but it's not useful as mpd will only return
 	   data to the client when we're done anyway. */
-	UPnPDirContent dirbuf;
-	if (!server.readDir(handle, tdirent.m_id.c_str(), dirbuf,
-			    error))
-		return false;
-
-	for (auto &dirent : dirbuf.objects) {
+	for (const auto &dirent : server.readDir(handle, tdirent.id.c_str()).objects) {
 		const std::string uri = PathTraitsUTF8::Build(base_uri,
 							      dirent.name.c_str());
-		if (!VisitObject(dirent, uri.c_str(),
-				 selection,
-				 visit_directory,
-				 visit_song, visit_playlist,
-				 error))
-			return false;
+		VisitObject(dirent, uri.c_str(),
+			    selection,
+			    visit_directory,
+			    visit_song, visit_playlist);
 	}
-
-	return true;
 }
 
 // Deal with the possibly multiple servers, call VisitServer if needed.
-bool
+void
 UpnpDatabase::Visit(const DatabaseSelection &selection,
 		    VisitDirectory visit_directory,
 		    VisitSong visit_song,
-		    VisitPlaylist visit_playlist,
-		    Error &error) const
+		    VisitPlaylist visit_playlist) const
 {
-	auto vpath = stringToTokens(selection.uri, "/", true);
+	auto vpath = stringToTokens(selection.uri, '/');
 	if (vpath.empty()) {
-		std::vector<ContentDirectoryService> servers;
-		if (!discovery->getDirServices(servers, error))
-			return false;
-
-		for (const auto &server : servers) {
+		for (const auto &server : discovery->GetDirectories()) {
 			if (visit_directory) {
 				const LightDirectory d(server.getFriendlyName(), 0);
-				if (!visit_directory(d, error))
-					return false;
+				visit_directory(d);
 			}
 
-			if (selection.recursive &&
-			    !VisitServer(server, vpath, selection,
-					 visit_directory, visit_song, visit_playlist,
-					 error))
-				return false;
+			if (selection.recursive)
+				VisitServer(server, vpath, selection,
+					    visit_directory, visit_song,
+					    visit_playlist);
 		}
 
-		return true;
+		return;
 	}
 
 	// We do have a path: the first element selects the server
 	std::string servername(std::move(vpath.front()));
 	vpath.pop_front();
 
-	ContentDirectoryService server;
-	if (!discovery->getServer(servername.c_str(), server, error))
-		return false;
-
-	return VisitServer(server, vpath, selection,
-			   visit_directory, visit_song, visit_playlist, error);
+	auto server = discovery->GetServer(servername.c_str());
+	VisitServer(server, vpath, selection,
+		    visit_directory, visit_song, visit_playlist);
 }
 
-bool
+void
 UpnpDatabase::VisitUniqueTags(const DatabaseSelection &selection,
-			      TagType tag, gcc_unused uint32_t group_mask,
-			      VisitTag visit_tag,
-			      Error &error) const
+			      TagType tag, gcc_unused tag_mask_t group_mask,
+			      VisitTag visit_tag) const
 {
 	// TODO: use group_mask
 
 	if (!visit_tag)
-		return true;
-
-	std::vector<ContentDirectoryService> servers;
-	if (!discovery->getDirServices(servers, error))
-		return false;
+		return;
 
 	std::set<std::string> values;
-	for (auto& server : servers) {
-		UPnPDirContent dirbuf;
-		if (!SearchSongs(server, rootid, selection, dirbuf, error))
-			return false;
+	for (auto& server : discovery->GetDirectories()) {
+		const auto dirbuf = SearchSongs(server, rootid, selection);
 
 		for (const auto &dirent : dirbuf.objects) {
 			if (dirent.type != UPnPDirObject::Type::ITEM ||
@@ -761,21 +636,18 @@ UpnpDatabase::VisitUniqueTags(const DatabaseSelection &selection,
 	for (const auto& value : values) {
 		TagBuilder builder;
 		builder.AddItem(tag, value.c_str());
-		if (!visit_tag(builder.Commit(), error))
-			return false;
+		visit_tag(builder.Commit());
 	}
-
-	return true;
 }
 
-bool
-UpnpDatabase::GetStats(const DatabaseSelection &,
-		       DatabaseStats &stats, Error &) const
+DatabaseStats
+UpnpDatabase::GetStats(const DatabaseSelection &) const
 {
 	/* Note: this gets called before the daemonizing so we can't
 	   reallyopen this would be a problem if we had real stats */
+	DatabaseStats stats;
 	stats.Clear();
-	return true;
+	return stats;
 }
 
 const DatabasePlugin upnp_db_plugin = {
