@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,38 +25,24 @@
 #include "config.h"
 #include "ExcludeList.hxx"
 #include "fs/Path.hxx"
-#include "fs/FileSystem.hxx"
-#include "util/StringUtil.hxx"
-#include "util/Domain.hxx"
+#include "fs/NarrowPath.hxx"
+#include "fs/io/TextFile.hxx"
+#include "system/Error.hxx"
 #include "Log.hxx"
 
+#include <stdexcept>
+
 #include <assert.h>
 #include <string.h>
-#include <errno.h>
-
-#ifdef HAVE_GLIB
-static constexpr Domain exclude_list_domain("exclude_list");
-#endif
 
 bool
 ExcludeList::LoadFile(Path path_fs)
-{
-#ifdef HAVE_GLIB
-	FILE *file = FOpen(path_fs, FOpenMode::ReadText);
-	if (file == nullptr) {
-		const int e = errno;
-		if (e != ENOENT) {
-			const auto path_utf8 = path_fs.ToUTF8();
-			FormatErrno(exclude_list_domain,
-				    "Failed to open %s",
-				    path_utf8.c_str());
-		}
+try {
+#ifdef HAVE_CLASS_GLOB
+	TextFile file(path_fs);
 
-		return false;
-	}
-
-	char line[1024];
-	while (fgets(line, sizeof(line), file) != nullptr) {
+	char *line;
+	while ((line = file.ReadLine()) != nullptr) {
 		char *p = strchr(line, '#');
 		if (p != nullptr)
 			*p = 0;
@@ -65,14 +51,19 @@ ExcludeList::LoadFile(Path path_fs)
 		if (*p != 0)
 			patterns.emplace_front(p);
 	}
-
-	fclose(file);
 #else
-	// TODO: implement
+	/* not implemented */
 	(void)path_fs;
 #endif
 
 	return true;
+} catch (const std::system_error &e) {
+	if (!IsFileNotFound(e))
+		LogError(e);
+	return false;
+} catch (const std::exception &e) {
+	LogError(e);
+	return false;
 }
 
 bool
@@ -82,12 +73,22 @@ ExcludeList::Check(Path name_fs) const
 
 	/* XXX include full path name in check */
 
-#ifdef HAVE_GLIB
-	for (const auto &i : patterns)
-		if (i.Check(name_fs.c_str()))
+#ifdef HAVE_CLASS_GLOB
+	if (parent != nullptr) {
+		if (parent->Check(name_fs)) {
 			return true;
+		}
+	}
+
+	for (const auto &i : patterns) {
+		try {
+			if (i.Check(NarrowPath(name_fs).c_str()))
+				return true;
+		} catch (const std::runtime_error &) {
+		}
+	}
 #else
-	// TODO: implement
+	/* not implemented */
 	(void)name_fs;
 #endif
 
