@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,12 +32,14 @@
 #include "FfmpegMetaData.hxx"
 #include "FfmpegIo.hxx"
 #include "pcm/Interleave.hxx"
+#include "tag/TagBuilder.hxx"
 #include "tag/TagHandler.hxx"
+#include "tag/ReplayGain.hxx"
+#include "tag/MixRamp.hxx"
 #include "input/InputStream.hxx"
 #include "CheckAudioFormat.hxx"
 #include "util/ScopeExit.hxx"
 #include "util/ConstBuffer.hxx"
-#include "util/Error.hxx"
 #include "LogV.hxx"
 
 extern "C" {
@@ -45,7 +47,6 @@ extern "C" {
 #include <libavformat/avformat.h>
 #include <libavformat/avio.h>
 #include <libavutil/avutil.h>
-#include <libavutil/log.h>
 
 #if LIBAVUTIL_VERSION_MAJOR >= 53
 #include <libavutil/frame.h>
@@ -55,48 +56,70 @@ extern "C" {
 #include <assert.h>
 #include <string.h>
 
+/**
+ * Muxer options to be passed to avformat_open_input().
+ */
+static AVDictionary *avformat_options = nullptr;
+
 static AVFormatContext *
 FfmpegOpenInput(AVIOContext *pb,
 		const char *filename,
-		AVInputFormat *fmt,
-		Error &error)
+		AVInputFormat *fmt)
 {
 	AVFormatContext *context = avformat_alloc_context();
-	if (context == nullptr) {
-		error.Set(ffmpeg_domain, "Out of memory");
-		return nullptr;
-	}
+	if (context == nullptr)
+		throw std::runtime_error("avformat_alloc_context() failed");
 
 	context->pb = pb;
 
-	int err = avformat_open_input(&context, filename, fmt, nullptr);
-	if (err < 0) {
-		SetFfmpegError(error, err, "avformat_open_input() failed");
-		return nullptr;
-	}
+	AVDictionary *options = nullptr;
+	AtScopeExit(&options) { av_dict_free(&options); };
+	av_dict_copy(&options, avformat_options, 0);
+
+	int err = avformat_open_input(&context, filename, fmt, &options);
+	if (err < 0)
+		throw MakeFfmpegError(err, "avformat_open_input() failed");
 
 	return context;
 }
 
 static bool
-ffmpeg_init(gcc_unused const config_param &param)
+ffmpeg_init(const ConfigBlock &block)
 {
 	FfmpegInit();
+
+	static constexpr const char *option_names[] = {
+		"probesize",
+		"analyzeduration",
+	};
+
+	for (const char *name : option_names) {
+		const char *value = block.GetBlockValue(name);
+		if (value != nullptr)
+			av_dict_set(&avformat_options, name, value, 0);
+	}
+
 	return true;
 }
 
+static void
+ffmpeg_finish()
+{
+	av_dict_free(&avformat_options);
+}
+
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 25, 0) /* FFmpeg 3.1 */
 
 gcc_pure
 static const AVCodecParameters &
-GetCodecParameters(const AVStream &stream)
+GetCodecParameters(const AVStream &stream) noexcept
 {
 	return *stream.codecpar;
 }
 
 gcc_pure
 static AVSampleFormat
-GetSampleFormat(const AVCodecParameters &codec_params)
+GetSampleFormat(const AVCodecParameters &codec_params) noexcept
 {
 	return AVSampleFormat(codec_params.format);
 }
@@ -105,14 +128,14 @@ GetSampleFormat(const AVCodecParameters &codec_params)
 
 gcc_pure
 static const AVCodecContext &
-GetCodecParameters(const AVStream &stream)
+GetCodecParameters(const AVStream &stream) noexcept
 {
 	return *stream.codec;
 }
 
 gcc_pure
 static AVSampleFormat
-GetSampleFormat(const AVCodecContext &codec_context)
+GetSampleFormat(const AVCodecContext &codec_context) noexcept
 {
 	return codec_context.sample_fmt;
 }
@@ -121,14 +144,14 @@ GetSampleFormat(const AVCodecContext &codec_context)
 
 gcc_pure
 static bool
-IsAudio(const AVStream &stream)
+IsAudio(const AVStream &stream) noexcept
 {
 	return GetCodecParameters(stream).codec_type == AVMEDIA_TYPE_AUDIO;
 }
 
 gcc_pure
 static int
-ffmpeg_find_audio_stream(const AVFormatContext &format_context)
+ffmpeg_find_audio_stream(const AVFormatContext &format_context) noexcept
 {
 	for (unsigned i = 0; i < format_context.nb_streams; ++i)
 		if (IsAudio(*format_context.streams[i]))
@@ -151,12 +174,13 @@ start_time_fallback(const AVStream &stream)
 
 /**
  * Copy PCM data from a non-empty AVFrame to an interleaved buffer.
+ *
+ * Throws #std::exception on error.
  */
 static ConstBuffer<void>
 copy_interleave_frame(const AVCodecContext &codec_context,
 		      const AVFrame &frame,
-		      FfmpegBuffer &global_buffer,
-		      Error &error)
+		      FfmpegBuffer &global_buffer)
 {
 	assert(frame.nb_samples > 0);
 
@@ -167,20 +191,16 @@ copy_interleave_frame(const AVCodecContext &codec_context,
 					   frame.nb_samples,
 					   codec_context.sample_fmt, 1);
 	assert(data_size != 0);
-	if (data_size < 0) {
-		SetFfmpegError(error, data_size);
-		return 0;
-	}
+	if (data_size < 0)
+		throw MakeFfmpegError(data_size);
 
 	void *output_buffer;
 	if (av_sample_fmt_is_planar(codec_context.sample_fmt) &&
 	    codec_context.channels > 1) {
 		output_buffer = global_buffer.GetT<uint8_t>(data_size);
-		if (output_buffer == nullptr) {
+		if (output_buffer == nullptr)
 			/* Not enough memory - shouldn't happen */
-			error.SetErrno(ENOMEM);
-			return 0;
-		}
+			throw std::bad_alloc();
 
 		PcmInterleave(output_buffer,
 			      ConstBuffer<const void *>((const void *const*)frame.extended_data,
@@ -200,7 +220,7 @@ copy_interleave_frame(const AVCodecContext &codec_context,
  */
 gcc_pure
 static int64_t
-StreamRelativePts(const AVPacket &packet, const AVStream &stream)
+StreamRelativePts(const AVPacket &packet, const AVStream &stream) noexcept
 {
 	auto pts = packet.pts;
 	if (pts < 0 || pts == int64_t(AV_NOPTS_VALUE))
@@ -217,28 +237,30 @@ StreamRelativePts(const AVPacket &packet, const AVStream &stream)
 gcc_pure
 static uint64_t
 PtsToPcmFrame(uint64_t pts, const AVStream &stream,
-	      const AVCodecContext &codec_context)
+	      const AVCodecContext &codec_context) noexcept
 {
 	return av_rescale_q(pts, stream.time_base, codec_context.time_base);
 }
 
 /**
- * Invoke decoder_data() with the contents of an #AVFrame.
+ * Invoke DecoderClient::SubmitData() with the contents of an
+ * #AVFrame.
  */
 static DecoderCommand
-FfmpegSendFrame(Decoder &decoder, InputStream &is,
+FfmpegSendFrame(DecoderClient &client, InputStream &is,
 		AVCodecContext &codec_context,
 		const AVFrame &frame,
 		size_t &skip_bytes,
 		FfmpegBuffer &buffer)
 {
-	Error error;
-	auto output_buffer =
-		copy_interleave_frame(codec_context, frame,
-				      buffer, error);
-	if (output_buffer.IsNull()) {
+	ConstBuffer<void> output_buffer;
+
+	try {
+		output_buffer = copy_interleave_frame(codec_context, frame,
+						      buffer);
+	} catch (const std::exception e) {
 		/* this must be a serious error, e.g. OOM */
-		LogError(error);
+		LogError(e);
 		return DecoderCommand::STOP;
 	}
 
@@ -254,15 +276,15 @@ FfmpegSendFrame(Decoder &decoder, InputStream &is,
 		skip_bytes = 0;
 	}
 
-	return decoder_data(decoder, is,
-			    output_buffer.data, output_buffer.size,
-			    codec_context.bit_rate / 1000);
+	return client.SubmitData(is,
+				 output_buffer.data, output_buffer.size,
+				 codec_context.bit_rate / 1000);
 }
 
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 37, 0)
 
 static DecoderCommand
-FfmpegReceiveFrames(Decoder &decoder, InputStream &is,
+FfmpegReceiveFrames(DecoderClient &client, InputStream &is,
 		    AVCodecContext &codec_context,
 		    AVFrame &frame,
 		    size_t &skip_bytes,
@@ -275,7 +297,7 @@ FfmpegReceiveFrames(Decoder &decoder, InputStream &is,
 		int err = avcodec_receive_frame(&codec_context, &frame);
 		switch (err) {
 		case 0:
-			cmd = FfmpegSendFrame(decoder, is, codec_context,
+			cmd = FfmpegSendFrame(client, is, codec_context,
 					      frame, skip_bytes,
 					      buffer);
 			if (cmd != DecoderCommand::NONE)
@@ -316,7 +338,7 @@ FfmpegReceiveFrames(Decoder &decoder, InputStream &is,
  * desired time stamp has been reached
  */
 static DecoderCommand
-ffmpeg_send_packet(Decoder &decoder, InputStream &is,
+ffmpeg_send_packet(DecoderClient &client, InputStream &is,
 		   AVPacket &&packet,
 		   AVCodecContext &codec_context,
 		   const AVStream &stream,
@@ -334,9 +356,8 @@ ffmpeg_send_packet(Decoder &decoder, InputStream &is,
 			if (cur_frame < min_frame)
 				skip_bytes = pcm_frame_size * (min_frame - cur_frame);
 		} else
-			decoder_timestamp(decoder,
-					  FfmpegTimeToDouble(pts,
-							     stream.time_base));
+			client.SubmitTimestamp(FfmpegTimeToDouble(pts,
+								  stream.time_base));
 	}
 
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(57, 37, 0)
@@ -362,7 +383,7 @@ ffmpeg_send_packet(Decoder &decoder, InputStream &is,
 		return DecoderCommand::NONE;
 	}
 
-	auto cmd = FfmpegReceiveFrames(decoder, is, codec_context,
+	auto cmd = FfmpegReceiveFrames(client, is, codec_context,
 				       frame,
 				       skip_bytes, buffer, eof);
 
@@ -387,7 +408,7 @@ ffmpeg_send_packet(Decoder &decoder, InputStream &is,
 		if (!got_frame || frame.nb_samples <= 0)
 			continue;
 
-		cmd = FfmpegSendFrame(decoder, is, codec_context,
+		cmd = FfmpegSendFrame(client, is, codec_context,
 				      frame, skip_bytes,
 				      buffer);
 	}
@@ -397,7 +418,7 @@ ffmpeg_send_packet(Decoder &decoder, InputStream &is,
 }
 
 static DecoderCommand
-ffmpeg_send_packet(Decoder &decoder, InputStream &is,
+ffmpeg_send_packet(DecoderClient &client, InputStream &is,
 		   const AVPacket &packet,
 		   AVCodecContext &codec_context,
 		   const AVStream &stream,
@@ -405,7 +426,7 @@ ffmpeg_send_packet(Decoder &decoder, InputStream &is,
 		   uint64_t min_frame, size_t pcm_frame_size,
 		   FfmpegBuffer &buffer)
 {
-	return ffmpeg_send_packet(decoder, is,
+	return ffmpeg_send_packet(client, is,
 				  /* copy the AVPacket, because FFmpeg
 				     < 3.0 requires this */
 				  AVPacket(packet),
@@ -416,7 +437,7 @@ ffmpeg_send_packet(Decoder &decoder, InputStream &is,
 
 gcc_const
 static SampleFormat
-ffmpeg_sample_format(enum AVSampleFormat sample_fmt)
+ffmpeg_sample_format(enum AVSampleFormat sample_fmt) noexcept
 {
 	switch (sample_fmt) {
 	case AV_SAMPLE_FMT_S16:
@@ -450,16 +471,22 @@ ffmpeg_sample_format(enum AVSampleFormat sample_fmt)
 }
 
 static AVInputFormat *
-ffmpeg_probe(Decoder *decoder, InputStream &is)
+ffmpeg_probe(DecoderClient *client, InputStream &is)
 {
 	constexpr size_t BUFFER_SIZE = 16384;
 	constexpr size_t PADDING = 16;
 
 	unsigned char buffer[BUFFER_SIZE];
-	size_t nbytes = decoder_read(decoder, is, buffer, BUFFER_SIZE);
-	if (nbytes <= PADDING || !is.LockRewind(IgnoreError()))
+	size_t nbytes = decoder_read(client, is, buffer, BUFFER_SIZE);
+	if (nbytes <= PADDING)
 		return nullptr;
 
+	try {
+		is.LockRewind();
+	} catch (const std::runtime_error &) {
+		return nullptr;
+	}
+
 	/* some ffmpeg parsers (e.g. ac3_parser.c) read a few bytes
 	   beyond the declared buffer limit, which makes valgrind
 	   angry; this workaround removes some padding from the buffer
@@ -493,7 +520,111 @@ ffmpeg_probe(Decoder *decoder, InputStream &is)
 }
 
 static void
-FfmpegDecode(Decoder &decoder, InputStream &input,
+FfmpegParseMetaData(AVDictionary &dict, ReplayGainInfo &rg, MixRampInfo &mr)
+{
+	AVDictionaryEntry *i = nullptr;
+
+	while ((i = av_dict_get(&dict, "", i,
+				AV_DICT_IGNORE_SUFFIX)) != nullptr) {
+		const char *name = i->key;
+		const char *value = i->value;
+
+		if (!ParseReplayGainTag(rg, name, value))
+			ParseMixRampTag(mr, name, value);
+	}
+}
+
+static void
+FfmpegParseMetaData(const AVStream &stream,
+		    ReplayGainInfo &rg, MixRampInfo &mr)
+{
+	FfmpegParseMetaData(*stream.metadata, rg, mr);
+}
+
+static void
+FfmpegParseMetaData(const AVFormatContext &format_context, int audio_stream,
+		    ReplayGainInfo &rg, MixRampInfo &mr)
+{
+	assert(audio_stream >= 0);
+
+	FfmpegParseMetaData(*format_context.metadata, rg, mr);
+	FfmpegParseMetaData(*format_context.streams[audio_stream],
+				    rg, mr);
+}
+
+static void
+FfmpegParseMetaData(DecoderClient &client,
+		    const AVFormatContext &format_context, int audio_stream)
+{
+	ReplayGainInfo rg;
+	rg.Clear();
+
+	MixRampInfo mr;
+	mr.Clear();
+
+	FfmpegParseMetaData(format_context, audio_stream, rg, mr);
+
+	if (rg.IsDefined())
+		client.SubmitReplayGain(&rg);
+
+	if (mr.IsDefined())
+		client.SubmitMixRamp(std::move(mr));
+}
+
+static void
+FfmpegScanMetadata(const AVStream &stream,
+		   const TagHandler &handler, void *handler_ctx)
+{
+	FfmpegScanDictionary(stream.metadata, handler, handler_ctx);
+}
+
+static void
+FfmpegScanMetadata(const AVFormatContext &format_context, int audio_stream,
+		   const TagHandler &handler, void *handler_ctx)
+{
+	assert(audio_stream >= 0);
+
+	FfmpegScanDictionary(format_context.metadata, handler, handler_ctx);
+	FfmpegScanMetadata(*format_context.streams[audio_stream],
+			   handler, handler_ctx);
+}
+
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(56, 1, 0)
+
+static void
+FfmpegScanTag(const AVFormatContext &format_context, int audio_stream,
+	      TagBuilder &tag)
+{
+	FfmpegScanMetadata(format_context, audio_stream,
+			   full_tag_handler, &tag);
+}
+
+/**
+ * Check if a new stream tag was received and pass it to
+ * DecoderClient::SubmitTag().
+ */
+static void
+FfmpegCheckTag(DecoderClient &client, InputStream &is,
+	       AVFormatContext &format_context, int audio_stream)
+{
+	AVStream &stream = *format_context.streams[audio_stream];
+	if ((stream.event_flags & AVSTREAM_EVENT_FLAG_METADATA_UPDATED) == 0)
+		/* no new metadata */
+		return;
+
+	/* clear the flag */
+	stream.event_flags &= ~AVSTREAM_EVENT_FLAG_METADATA_UPDATED;
+
+	TagBuilder tag;
+	FfmpegScanTag(format_context, audio_stream, tag);
+	if (!tag.IsEmpty())
+		client.SubmitTag(is, tag.Commit());
+}
+
+#endif
+
+static void
+FfmpegDecode(DecoderClient &client, InputStream &input,
 	     AVFormatContext &format_context)
 {
 	const int find_result =
@@ -559,15 +690,9 @@ FfmpegDecode(Decoder &decoder, InputStream &input,
 		return;
 	}
 
-	Error error;
-	AudioFormat audio_format;
-	if (!audio_format_init_checked(audio_format,
-				       codec_params.sample_rate,
-				       sample_format,
-				       codec_params.channels, error)) {
-		LogError(error);
-		return;
-	}
+	const auto audio_format = CheckAudioFormat(codec_params.sample_rate,
+						   sample_format,
+						   codec_params.channels);
 
 	/* the audio format must be read from AVCodecContext by now,
 	   because avcodec_open() has been demonstrated to fill bogus
@@ -589,8 +714,9 @@ FfmpegDecode(Decoder &decoder, InputStream &input,
 	const SignedSongTime total_time =
 		FromFfmpegTimeChecked(av_stream.duration, av_stream.time_base);
 
-	decoder_initialized(decoder, audio_format,
-			    input.IsSeekable(), total_time);
+	client.Ready(audio_format, input.IsSeekable(), total_time);
+
+	FfmpegParseMetaData(client, format_context, audio_stream);
 
 #if LIBAVUTIL_VERSION_MAJOR >= 53
 	AVFrame *frame = av_frame_alloc();
@@ -616,11 +742,11 @@ FfmpegDecode(Decoder &decoder, InputStream &input,
 
 	uint64_t min_frame = 0;
 
-	DecoderCommand cmd = decoder_get_command(decoder);
+	DecoderCommand cmd = client.GetCommand();
 	while (cmd != DecoderCommand::STOP) {
 		if (cmd == DecoderCommand::SEEK) {
 			int64_t where =
-				ToFfmpegTime(decoder_seek_time(decoder),
+				ToFfmpegTime(client.GetSeekTime(),
 					     av_stream.time_base) +
 				start_time_fallback(av_stream);
 
@@ -629,11 +755,11 @@ FfmpegDecode(Decoder &decoder, InputStream &input,
 			   stamp, not after */
 			if (av_seek_frame(&format_context, audio_stream, where,
 					  AVSEEK_FLAG_ANY|AVSEEK_FLAG_BACKWARD) < 0)
-				decoder_seek_error(decoder);
+				client.SeekError();
 			else {
 				avcodec_flush_buffers(codec_context);
-				min_frame = decoder_seek_where_frame(decoder);
-				decoder_command_finished(decoder);
+				min_frame = client.GetSeekFrame();
+				client.CommandFinished();
 			}
 		}
 
@@ -642,8 +768,12 @@ FfmpegDecode(Decoder &decoder, InputStream &input,
 			/* end of file */
 			break;
 
+#if LIBAVFORMAT_VERSION_INT >= AV_VERSION_INT(56, 1, 0)
+		FfmpegCheckTag(client, input, format_context, audio_stream);
+#endif
+
 		if (packet.size > 0 && packet.stream_index == audio_stream) {
-			cmd = ffmpeg_send_packet(decoder, input,
+			cmd = ffmpeg_send_packet(client, input,
 						 packet,
 						 *codec_context,
 						 av_stream,
@@ -652,7 +782,7 @@ FfmpegDecode(Decoder &decoder, InputStream &input,
 						 interleaved_buffer);
 			min_frame = 0;
 		} else
-			cmd = decoder_get_command(decoder);
+			cmd = client.GetCommand();
 
 #if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(56, 25, 100)
 		av_packet_unref(&packet);
@@ -663,26 +793,27 @@ FfmpegDecode(Decoder &decoder, InputStream &input,
 }
 
 static void
-ffmpeg_decode(Decoder &decoder, InputStream &input)
+ffmpeg_decode(DecoderClient &client, InputStream &input)
 {
-	AVInputFormat *input_format = ffmpeg_probe(&decoder, input);
+	AVInputFormat *input_format = ffmpeg_probe(&client, input);
 	if (input_format == nullptr)
 		return;
 
 	FormatDebug(ffmpeg_domain, "detected input format '%s' (%s)",
 		    input_format->name, input_format->long_name);
 
-	AvioStream stream(&decoder, input);
+	AvioStream stream(&client, input);
 	if (!stream.Open()) {
 		LogError(ffmpeg_domain, "Failed to open stream");
 		return;
 	}
 
-	Error error;
-	AVFormatContext *format_context =
-		FfmpegOpenInput(stream.io, input.GetURI(), input_format, error);
-	if (format_context == nullptr) {
-		LogError(error);
+	AVFormatContext *format_context;
+	try {
+		format_context =FfmpegOpenInput(stream.io, input.GetURI(),
+						input_format);
+	} catch (const std::runtime_error &e) {
+		LogError(e);
 		return;
 	}
 
@@ -690,37 +821,36 @@ ffmpeg_decode(Decoder &decoder, InputStream &input)
 		avformat_close_input(&format_context);
 	};
 
-	FfmpegDecode(decoder, input, *format_context);
+	FfmpegDecode(client, input, *format_context);
 }
 
 static bool
 FfmpegScanStream(AVFormatContext &format_context,
-		 const struct tag_handler &handler, void *handler_ctx)
+		 const TagHandler &handler, void *handler_ctx)
 {
 	const int find_result =
 		avformat_find_stream_info(&format_context, nullptr);
 	if (find_result < 0)
 		return false;
 
-	if (format_context.duration != (int64_t)AV_NOPTS_VALUE) {
-		const auto duration =
-			SongTime::FromScale<uint64_t>(format_context.duration,
-						      AV_TIME_BASE);
-		tag_handler_invoke_duration(&handler, handler_ctx, duration);
-	}
+	const int audio_stream = ffmpeg_find_audio_stream(format_context);
+	if (audio_stream < 0)
+		return false;
 
-	FfmpegScanDictionary(format_context.metadata, &handler, handler_ctx);
-	int idx = ffmpeg_find_audio_stream(format_context);
-	if (idx >= 0)
-		FfmpegScanDictionary(format_context.streams[idx]->metadata,
-				     &handler, handler_ctx);
+	const AVStream &stream = *format_context.streams[audio_stream];
+	if (stream.duration != (int64_t)AV_NOPTS_VALUE)
+		tag_handler_invoke_duration(handler, handler_ctx,
+					    FromFfmpegTime(stream.duration,
+							   stream.time_base));
+
+	FfmpegScanMetadata(format_context, audio_stream, handler, handler_ctx);
 
 	return true;
 }
 
 static bool
 ffmpeg_scan_stream(InputStream &is,
-		   const struct tag_handler *handler, void *handler_ctx)
+		   const TagHandler &handler, void *handler_ctx)
 {
 	AVInputFormat *input_format = ffmpeg_probe(nullptr, is);
 	if (input_format == nullptr)
@@ -730,17 +860,18 @@ ffmpeg_scan_stream(InputStream &is,
 	if (!stream.Open())
 		return false;
 
-	AVFormatContext *f =
-		FfmpegOpenInput(stream.io, is.GetURI(), input_format,
-				IgnoreError());
-	if (f == nullptr)
+	AVFormatContext *f;
+	try {
+		f = FfmpegOpenInput(stream.io, is.GetURI(), input_format);
+	} catch (const std::runtime_error &) {
 		return false;
+	}
 
 	AtScopeExit(&f) {
 		avformat_close_input(&f);
 	};
 
-	return FfmpegScanStream(*f, *handler, handler_ctx);
+	return FfmpegScanStream(*f, handler, handler_ctx);
 }
 
 /**
@@ -863,7 +994,7 @@ static const char *const ffmpeg_mime_types[] = {
 const struct DecoderPlugin ffmpeg_decoder_plugin = {
 	"ffmpeg",
 	ffmpeg_init,
-	nullptr,
+	ffmpeg_finish,
 	ffmpeg_decode,
 	nullptr,
 	nullptr,
