@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,9 @@
 #include "AudioFormat.hxx"
 #include "poison.h"
 
+#include <stdexcept>
+#include <memory>
+
 #include <assert.h>
 
 class ConvertFilter final : public Filter {
@@ -38,108 +41,110 @@ class ConvertFilter final : public Filter {
 	AudioFormat in_audio_format;
 
 	/**
-	 * The output audio format; the consumer of this plugin
-	 * expects PCM data in this format.
-	 *
-	 * If this is AudioFormat::Undefined(), then the #PcmConvert
-	 * attribute is not open.  This can mean that Set() has failed
-	 * or that no conversion is necessary.
+	 * This object is only "open" if #in_audio_format !=
+	 * #out_audio_format.
 	 */
-	AudioFormat out_audio_format;
-
-	Manual<PcmConvert> state;
+	PcmConvert state;
 
 public:
-	bool Set(const AudioFormat &_out_audio_format, Error &error);
+	ConvertFilter(const AudioFormat &audio_format);
+	~ConvertFilter();
 
-	virtual AudioFormat Open(AudioFormat &af, Error &error) override;
-	virtual void Close() override;
-	virtual ConstBuffer<void> FilterPCM(ConstBuffer<void> src,
-					    Error &error) override;
+	void Set(const AudioFormat &_out_audio_format);
+
+	void Reset() override {
+		state.Reset();
+	}
+
+	ConstBuffer<void> FilterPCM(ConstBuffer<void> src) override;
 };
 
-static Filter *
-convert_filter_init(gcc_unused const config_param &param,
-		    gcc_unused Error &error)
+class PreparedConvertFilter final : public PreparedFilter {
+public:
+	Filter *Open(AudioFormat &af) override;
+};
+
+static PreparedFilter *
+convert_filter_init(gcc_unused const ConfigBlock &block)
 {
-	return new ConvertFilter();
+	return new PreparedConvertFilter();
 }
 
-bool
-ConvertFilter::Set(const AudioFormat &_out_audio_format, Error &error)
+void
+ConvertFilter::Set(const AudioFormat &_out_audio_format)
 {
 	assert(in_audio_format.IsValid());
 	assert(_out_audio_format.IsValid());
 
 	if (_out_audio_format == out_audio_format)
 		/* no change */
-		return true;
+		return;
 
-	if (out_audio_format.IsValid()) {
-		out_audio_format.Clear();
-		state->Close();
+	if (out_audio_format != in_audio_format) {
+		out_audio_format = in_audio_format;
+		state.Close();
 	}
 
 	if (_out_audio_format == in_audio_format)
 		/* optimized special case: no-op */
-		return true;
+		return;
 
-	if (!state->Open(in_audio_format, _out_audio_format, error))
-		return false;
+	state.Open(in_audio_format, _out_audio_format);
 
 	out_audio_format = _out_audio_format;
-	return true;
 }
 
-AudioFormat
-ConvertFilter::Open(AudioFormat &audio_format, gcc_unused Error &error)
+ConvertFilter::ConvertFilter(const AudioFormat &audio_format)
+	:Filter(audio_format), in_audio_format(audio_format)
+{
+}
+
+Filter *
+PreparedConvertFilter::Open(AudioFormat &audio_format)
 {
 	assert(audio_format.IsValid());
 
-	in_audio_format = audio_format;
-	out_audio_format.Clear();
-
-	state.Construct();
-
-	return in_audio_format;
+	return new ConvertFilter(audio_format);
 }
 
-void
-ConvertFilter::Close()
+ConvertFilter::~ConvertFilter()
 {
 	assert(in_audio_format.IsValid());
 
-	if (out_audio_format.IsValid())
-		state->Close();
-
-	state.Destruct();
-
-	poison_undefined(&in_audio_format, sizeof(in_audio_format));
-	poison_undefined(&out_audio_format, sizeof(out_audio_format));
+	if (out_audio_format != in_audio_format)
+		state.Close();
 }
 
 ConstBuffer<void>
-ConvertFilter::FilterPCM(ConstBuffer<void> src, Error &error)
+ConvertFilter::FilterPCM(ConstBuffer<void> src)
 {
 	assert(in_audio_format.IsValid());
 
-	if (!out_audio_format.IsValid())
+	if (out_audio_format == in_audio_format)
 		/* optimized special case: no-op */
 		return src;
 
-	return state->Convert(src, error);
+	return state.Convert(src);
 }
 
-const struct filter_plugin convert_filter_plugin = {
+const FilterPlugin convert_filter_plugin = {
 	"convert",
 	convert_filter_init,
 };
 
-bool
-convert_filter_set(Filter *_filter, AudioFormat out_audio_format,
-		   Error &error)
+Filter *
+convert_filter_new(const AudioFormat in_audio_format,
+		   const AudioFormat out_audio_format)
+{
+	std::unique_ptr<ConvertFilter> filter(new ConvertFilter(in_audio_format));
+	filter->Set(out_audio_format);
+	return filter.release();
+}
+
+void
+convert_filter_set(Filter *_filter, AudioFormat out_audio_format)
 {
 	ConvertFilter *filter = (ConvertFilter *)_filter;
 
-	return filter->Set(out_audio_format, error);
+	filter->Set(out_audio_format);
 }
