@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,14 +30,10 @@
 #include "thread/Cond.hxx"
 #include "thread/Thread.hxx"
 #include "thread/Name.hxx"
-#include "util/Macros.hxx"
-#include "util/Domain.hxx"
-#include "util/Error.hxx"
 #include "Log.hxx"
 
 #include <libsmbclient.h>
 
-#include <list>
 #include <algorithm>
 
 class SmbclientNeighborExplorer final : public NeighborExplorer {
@@ -76,7 +72,7 @@ public:
 		:NeighborExplorer(_listener) {}
 
 	/* virtual methods from class NeighborExplorer */
-	virtual bool Open(Error &error) override;
+	void Open() override;
 	virtual void Close() override;
 	virtual List GetList() const override;
 
@@ -86,11 +82,11 @@ private:
 	static void ThreadFunc(void *ctx);
 };
 
-bool
-SmbclientNeighborExplorer::Open(Error &error)
+void
+SmbclientNeighborExplorer::Open()
 {
 	quit = false;
-	return thread.Start(ThreadFunc, this, error);
+	thread.Start(ThreadFunc, this);
 }
 
 void
@@ -107,7 +103,7 @@ SmbclientNeighborExplorer::Close()
 NeighborExplorer::List
 SmbclientNeighborExplorer::GetList() const
 {
-	const ScopeLock protect(mutex);
+	const std::lock_guard<Mutex> protect(mutex);
 	/*
 	List list;
 	for (const auto &i : servers)
@@ -173,10 +169,10 @@ ReadServers(NeighborExplorer::List &list, const char *uri)
 
 gcc_pure
 static NeighborExplorer::List
-DetectServers()
+DetectServers() noexcept
 {
 	NeighborExplorer::List list;
-	const ScopeLock protect(smbclient_mutex);
+	const std::lock_guard<Mutex> protect(smbclient_mutex);
 	ReadServers(list, "smb://");
 	return list;
 }
@@ -185,7 +181,7 @@ gcc_pure
 static NeighborExplorer::List::const_iterator
 FindBeforeServerByURI(NeighborExplorer::List::const_iterator prev,
 		      NeighborExplorer::List::const_iterator end,
-		      const std::string &uri)
+		      const std::string &uri) noexcept
 {
 	for (auto i = std::next(prev); i != end; prev = i, i = std::next(prev))
 		if (i->uri == uri)
@@ -255,7 +251,7 @@ SmbclientNeighborExplorer::ThreadFunc()
 			break;
 
 		// TODO: sleep for how long?
-		cond.timed_wait(mutex, 10000);
+		cond.timed_wait(mutex, std::chrono::seconds(10));
 	}
 
 	mutex.unlock();
@@ -273,11 +269,9 @@ SmbclientNeighborExplorer::ThreadFunc(void *ctx)
 static NeighborExplorer *
 smbclient_neighbor_create(gcc_unused EventLoop &loop,
 			  NeighborListener &listener,
-			  gcc_unused const config_param &param,
-			  gcc_unused Error &error)
+			  gcc_unused const ConfigBlock &block)
 {
-	if (!SmbclientInit(error))
-		return nullptr;
+	SmbclientInit();
 
 	return new SmbclientNeighborExplorer(listener);
 }
