@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 The Music Player Daemon Project
+ * Copyright 2003-2017 The Music Player Daemon Project
  * http://www.musicpd.org
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,10 +18,13 @@
  */
 
 #include "config.h"
-#include "system/Resolver.hxx"
-#include "util/Error.hxx"
+#include "net/Resolver.hxx"
+#include "net/ToString.hxx"
+#include "net/SocketAddress.hxx"
 #include "Log.hxx"
 
+#include <stdexcept>
+
 #ifdef WIN32
 #include <ws2tcpip.h>
 #include <winsock.h>
@@ -34,26 +37,23 @@
 #include <stdlib.h>
 
 int main(int argc, char **argv)
-{
+try {
 	if (argc != 2) {
 		fprintf(stderr, "Usage: run_resolver HOST\n");
 		return EXIT_FAILURE;
 	}
 
-	Error error;
 	struct addrinfo *ai =
-		resolve_host_port(argv[1], 80, AI_PASSIVE, SOCK_STREAM,
-				  error);
-	if (ai == NULL) {
-		LogError(error);
-		return EXIT_FAILURE;
-	}
+		resolve_host_port(argv[1], 80, AI_PASSIVE, SOCK_STREAM);
 
 	for (const struct addrinfo *i = ai; i != NULL; i = i->ai_next) {
-		const auto s = sockaddr_to_string(i->ai_addr, i->ai_addrlen);
+		const auto s = ToString({i->ai_addr, i->ai_addrlen});
 		printf("%s\n", s.c_str());
 	}
 
 	freeaddrinfo(ai);
 	return EXIT_SUCCESS;
+} catch (const std::runtime_error &e) {
+	LogError(e);
+	return EXIT_FAILURE;
 }
